c/* %W%      latest revision %G% %U% */
c/*	dimensionality of v(40) --> v(54) to correspond to addition
c/*     of f-ratios. Should not change anything else.
c/*	PJD 26/3/93
c/*
       subroutine GRAZPT (Kappa, v)
************************************************************************

c *** Computes the Graz potential (Schwartz, Zingl, Mathelisch, UNI Graz
c *** UTP 02/79 "Separable Nucleon-Nucleon Potentials and Minimal
c *** Relativity")

c *** Convention
c ***  v(1) = 1S0,  2 = 1D2,   3 = 1G4,  4 = 3P0,  5 = 3P1,  6 = 3P2,   7 = 3F2
c        8  = 3F3,  9 = 3F4,  10 = 3H4, 11 = 3H5, 12 = 3H6, 13 = 1S0,  14 = 1P1
c       15  = 1F3, 16 = 1H5,  17 = 3S1, 18 = 3D1, 19 = 3D2, 20 = 3D3,  21 = 3G3
c       22  = 3G4, 23 = 3G5,  24 = 1I6, 25 = 3J6, 26 = 3J7, 27 = 3I5,  28 = 3I6
c       29  = 3I7, 30 = 1J7,  31 = 3K7, 32 = 3K8
c ***

      implicit real*8 (a-h, i, k, o-z)
CCC
      dimension v(54)
CCC      dimension v(40)

c >>> FIRST EXECUTABLE STATEMENT <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      p = Kappa/197.33

c *** Last line because the parameters in the potentials are given in
c *** fermis squared.

c **** 3S1 ***

      v(17) = 1./( p**2 + 1.43**2 )

c **** 1S0 ****

      v(1)  = 1./( p**2 + 1.152**2 )
      v(13) = v(1)

c **** 1P1 ****

      anu1 = p * ( 1.0 + 1.0297 * p**2 )
      den1 = ( 1. + 4.2965 * p**2 ) * ( 1. + 0.0506 * p**2 )
     $       * ( 1. + 0.0493 * p**2 )
      v(14) = anu1/den1

c **** 3P0 ****

      anu2 = p * ( 1. + 4.3036 * p**2 )
      den2 = ( 1. + 4.3036 * p**2 ) * ( 1. + 4.0351 * p**2 )
     $       * ( 1. + 0.7292 * p**2 )
      v(4) = anu2/den2

c **** 3P1 *****

      anu3 = p * ( 1. + 0.9055 * p**2 )
      den3 = ( 1. + 3.6475 * p**2 ) * ( 1. + 0.0446 * p**2 )
     $       * ( 1. + 0.0412 * p**2 )
      v(5) = anu3/den3

c **** 3P2 ****

      anu4 = p * ( 1. + 0.1514 * p**2 )
      den4 = ( 1. + 0.4266 * p**2 ) * ( 1. + 0.0634 * p**2 )
     $       * ( 1 + 0.0634 * p**2 )
      v(6) = anu4/den4

c **** 3D2 ****

      anu5 = ( 1. + 0.7544 * p**2 ) * p**2
      den5 = ( 1. + 6.7807 * p**2 ) * ( 1. + 0.1583 * p**2 )
     $       * ( 1. + 0.1538 * p**2 ) * ( 1. + 0.0738 * p**2 )
      v(19) = anu5/den5

c **** 1D2 ****

      anu6 = ( 1. + 0.4897 * p**2 ) * p**2
      den6 = ( 1. + 5.0341 * p**2 ) * ( 1. + 0.1589 * p**2 )
     $       * ( 1. + 0.0705 * p**2 ) * ( 1. + 0.0169 * p**2 )
      v(2) = anu6/den6

c **** 3D1 and 3D3 ****

      v(18) = v(2)
      v(20) = v(2)

c **** 3F2, 3F3, 3F4, 1F3 ****

      pfac = p/( 1. + 0.5 * p**2 )
      v(7) = pfac * v(2)
      v(8) = v(7)
      v(9) = v(7)
      v(15) = v(7)

c **** 1G4, 3G3, 3G4, 3G5 ****

      v(3) = pfac * v(7)
      v(21) = v(3)
      v(22) = v(3)
      v(23) = v(3)

c **** 3H4, 3H5, 3H6, 1H5 ****

      v(10) = pfac * v(3)
      v(11) = v(10)
      v(12) = v(10)
      v(16) = v(10)

c **** 1I6, 3I5, 3I6, 3I7 ****

      v(24) = pfac * v(10)
      v(27) = v(24)
      v(28) = v(24)
      v(29) = v(24)

c **** 3J6, 3J7, 1J7 ****

      v(25) = pfac * v(24)
      v(26) = v(25)
      v(30) = v(25)

c **** 3K7, 3K8 ****

      v(31) = pfac * v(25)
      v(32) = v(31)

      RETURN
      end

